/*
 *  yuvframestofields -double the frame rate by replicating each field
 *
 *  Copyright (C) 2005, Jerome Cornet <jcornet@free.fr>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#define HAVE_STDINT_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <mjpegtools/yuv4mpeg.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>



int
main (int argc, char *argv[])
{
  int stream0 = STDIN_FILENO;	/* read from stdin */
  int outstream = STDOUT_FILENO;	/* output to stdout */
  int w, h;
  unsigned char *yuv_top[3], *yuv_bottom[3], *yuv_null[3];
  y4m_stream_info_t sinfo0;
  y4m_stream_info_t osinfo;
  y4m_frame_info_t finfo0;
  int err;
  y4m_ratio_t framerate;
  char c;
  int dominance = Y4M_ILACE_BOTTOM_FIRST;
  int force_processing = 0;
  int verbose = 0;
  int framenumber = 0;


  while ((c = getopt (argc, argv, "hvtf")) != -1)
    {
      switch (c)
	{
	case 'h':
	  {
	    mjpeg_log (LOG_INFO,
		       " -------------------------------------------------");
	    mjpeg_log (LOG_INFO,
		       "            Framerate halver/interlacer            ");
	    mjpeg_log (LOG_INFO,
		       " -------------------------------------------------");

	    mjpeg_log (LOG_INFO, " Usage ");

	    mjpeg_log (LOG_INFO, " -h: this message");
	    mjpeg_log (LOG_INFO, " -v: verbose output");
	    mjpeg_log (LOG_INFO,
		       " -t: force top field first (default=bottom)");
	    mjpeg_log (LOG_INFO, " -f: force processing interlaced material");
	    exit (0);
	  }
	  break;
	case 'v':
	  verbose = 1;
	  break;

	case 't':
	  dominance = Y4M_ILACE_TOP_FIRST;
	  break;
	case 'f':
	  force_processing = 1;
	  break;
	}
    }

  y4m_init_stream_info (&sinfo0);
  y4m_init_stream_info (&osinfo);
  y4m_init_frame_info (&finfo0);


  if ((err = y4m_read_stream_header (stream0, &sinfo0)) != Y4M_OK)
    mjpeg_error_exit1 ("Failed to input stream header:  %s errno=%d",
		       y4m_strerr (err), errno);

  if (y4m_si_get_plane_count (&sinfo0) != 3)
    mjpeg_error_exit1 ("Only 3-plane formats supported.");

  mjpeg_info ("Input stream parameters:");
  y4m_log_stream_info (LOG_INFO, "1> ", &sinfo0);

  w = y4m_si_get_width (&sinfo0);
  h = y4m_si_get_height (&sinfo0);

  y4m_copy_stream_info (&osinfo, &sinfo0);
  framerate = y4m_si_get_framerate (&sinfo0);
  y4m_ratio_reduce (&framerate);

  if ((y4m_si_get_interlace (&sinfo0) != Y4M_ILACE_NONE)
      && (force_processing != 1))
    mjpeg_error_exit1
      ("Sorry, source material is interlaced... use -f to force processing");

  y4m_si_set_interlace (&osinfo, dominance);

  framerate.d *= 2;
  y4m_ratio_reduce (&framerate);
  y4m_si_set_framerate (&osinfo, framerate);


  y4m_log_stream_info (LOG_INFO, "1> ", &osinfo);


  /* Allocate for worst case (4:4:4) */
  yuv_top[0] = (unsigned char *) malloc (w * h);
  yuv_top[1] = (unsigned char *) malloc (w * h);
  yuv_top[2] = (unsigned char *) malloc (w * h);
  yuv_bottom[0] = (unsigned char *) malloc (w * h);
  yuv_bottom[1] = (unsigned char *) malloc (w * h);
  yuv_bottom[2] = (unsigned char *) malloc (w * h);

  yuv_null[0] = (unsigned char *) malloc (w * h);
  yuv_null[1] = (unsigned char *) malloc (w * h);
  yuv_null[2] = (unsigned char *) malloc (w * h);


  if ((err = y4m_write_stream_header (outstream, &osinfo)) != Y4M_OK)
    mjpeg_error_exit1 ("Failed to write output header:  %s",
		       y4m_strerr (err));
  if (dominance == Y4M_ILACE_BOTTOM_FIRST)
    {
      while (((err =
	       y4m_read_fields (stream0, &sinfo0, &finfo0, yuv_null,
				yuv_bottom)) == Y4M_OK)
	     &&
	     ((err =
	       y4m_read_fields (stream0, &sinfo0, &finfo0, yuv_top,
				yuv_null)) == Y4M_OK)
	     &&
	     ((err =
	       y4m_write_fields (outstream, &osinfo, &finfo0, yuv_top,
				 yuv_bottom)) == Y4M_OK))
	{
	  if (verbose)
	    mjpeg_log (LOG_INFO, "Output Frame #%d", framenumber++);
	}
    }
  else
    {
      while (((err =
	       y4m_read_fields (stream0, &sinfo0, &finfo0, yuv_top,
				yuv_null)) == Y4M_OK)
	     &&
	     ((err =
	       y4m_read_fields (stream0, &sinfo0, &finfo0, yuv_null,
				yuv_bottom)) == Y4M_OK)
	     &&
	     ((err =
	       y4m_write_fields (outstream, &osinfo, &finfo0, yuv_top,
				 yuv_bottom)) == Y4M_OK))
	{
	}
    }
  if (err != Y4M_ERR_EOF)
    mjpeg_error_exit1 ("Some error somewhere:  %s", y4m_strerr (err));

  free (yuv_top[0]);
  free (yuv_top[1]);
  free (yuv_top[2]);
  free (yuv_bottom[0]);
  free (yuv_bottom[1]);
  free (yuv_bottom[2]);
  free (yuv_null[0]);
  free (yuv_null[1]);
  free (yuv_null[2]);

  y4m_fini_stream_info (&sinfo0);
  y4m_fini_stream_info (&osinfo);
  y4m_fini_frame_info (&finfo0);
  return 0;
}
